package com.fossgalaxy.games.ggj2017;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.Map;

import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;

import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.EntityType;
import com.fossgalaxy.games.rts.entity.ResourceType;
import com.fossgalaxy.games.rts.order.BuildOrder;
import com.fossgalaxy.games.rts.order.Order;

public class BuildAction extends DroneAction {
	private EntityType type;

	@ObjectDef("BuildAction")
	public BuildAction(EntityType type) {
		this.type = type;
	}
	
	@Override
	public void renderHints(Graphics2D g, GameState s, Entity actor) {
		
	}

	@Override
	public Order generateOrder(CubeCoordinate co, GameState s) {
		return new BuildOrder(type, co);
	}
	
	@Override
	public boolean isPossible(Entity entity, GameState s, CubeCoordinate co) {
		if (!super.isPossible(entity, s, co)) {
			return false;
		}
		
		//is the tile free?
		Entity entityAt = s.getEntityAt(co);
		if (entityAt != null) {
			return false;
		}
		
		//can we afford it?
		for (Map.Entry<ResourceType, Integer> costEntry : type.getCosts().entrySet()) {	
			int currVal = s.getResource(entity.getOwner(), costEntry.getKey());
			if (currVal < costEntry.getValue()) {
				return false;
			}	
		}
		
		//are we next to the space?
		int distance = s.getDistance(entity.getPos(), co);
		if (distance != 1) {
			return false;
		}
		
		return true;
	}

	@Override
	public boolean isPossible(Entity entity) {
		return true;
	}

	@Override
	public Color getHintColour() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Color getBorderColour() {
		// TODO Auto-generated method stub
		return null;
	}

	public String toString() {
		return "build "+type.getName();
	}
	
}
