package com.fossgalaxy.games.ggj2017;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Arrays;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.swing.*;

import com.fossgalaxy.games.rts.App;
import com.fossgalaxy.games.rts.GameDef;
import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.ai.Controller;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.EntityType;
import com.fossgalaxy.games.rts.io.SettingsIO;
import com.fossgalaxy.games.rts.order.Order;
import com.fossgalaxy.games.rts.order.OrderProcessor;
import com.fossgalaxy.games.rts.rules.Rule;
import com.fossgalaxy.games.rts.ui.GameAction;
import com.fossgalaxy.games.rts.ui.UIModel;
import com.fossgalaxy.games.rts.ui.GameView;
import com.fossgalaxy.games.rts.ui.KeyboardController;
import com.fossgalaxy.games.rts.ui.MouseController;
import com.fossgalaxy.games.rts.ui.UIController;

/**
 * Hello world!
 *
 */
public class JamApp 
{
	
	public static boolean isZoneOfControl(GameState state, Entity curr) {
		Collection<Entity> entities = state.getOwnedEntities(curr.getOwner());
		
		for (Entity e : entities) {
			Integer zoneDistance = e.getType().getProperty("controlSignal");
			if (zoneDistance != null) {
				int distance = state.getDistance(e.getPos(), curr.getPos());
				if (distance <= zoneDistance) {
					return true;
				}
			}
		}
		
		return false;
	}
	
    public static void main( String[] args )
    {

        //create the initial game state
        SettingsIO io = new SettingsIO("transmission.json");
        GameState state = io.buildGame("bigMap.json");
        
        GameDef def = io.getGameDef();

        //create the players
        //TODO clone, without it all moves WILL be applied twice.
        GameState playerState0 = new GameState(state);
        GameState playerState1 = new GameState(state);
        
        EntityType blueBase = null;
        for (EntityType t : io.getEntityTypes()) {
        	if (t.getName().equals("blue_base")) {
        		blueBase = t;
        		break;
        	}
        }
        
        Collection<GameAction> actions = getPossibleActions(blueBase);
        
        Controller[] players = new Controller[]{
                JamApp.buildUI(playerState0, "player 0", actions), JamApp.buildUI(playerState1, "player 1", actions)
        };

        //create the thing that's doing the turn tracking.
        OrderProcessor processor = new OrderProcessor(state, 2);
        int MAX_TURNS = 100;
        
        for (Rule rule : def.getRules()) {
        	processor.addRule(rule);
        }

        //turn loop
        //TODO this might be better inside the turn processor?
        for (int i=0; i<MAX_TURNS; i++) {
            processor.setupTurn();

            //figure out who's turn it is.
            int playerID = processor.getCurrentPlayer();
            Controller player = players[playerID];

            //process the controller's orders
            GameState curr = new GameState(state);
            Map<UUID, Order> orders = player.doTurn(playerID, curr);
            processor.doOrderBulk(orders);

            processor.finishTurn();
            
            if (processor.getWinner() != Rule.NO_WINNER) {
            	break;
            }
        }
        
        System.out.println("Game over, player won: "+processor.getWinner());

    }

    public static Collection<GameAction> getPossibleActions(EntityType base) {
    	return Arrays.asList(new MoveAction(), new RangeAttackAction(), new BuildAction(base));
    }
  
    
    public static Collection<GameAction> getGameActions(Entity entity, Collection<GameAction> actions) {
    	return actions.stream().filter(act -> act.isPossible(entity)).collect(Collectors.toList());
    }
    
    public static Controller buildUI(GameState state, String title, Collection<GameAction> possibleActions) {

        JFrame frame = new JFrame(title);
        frame.setPreferredSize(new Dimension(800, 600));
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //create the UI stuff
        UIModel model = new UIModel(state);
        GameView view = new GameView(model);
        JScrollPane viewport = new JScrollPane(view);
//        viewport.setView(view);
        view.addMouseListener(new MouseController(view, model));

        view.addKeyListener(new KeyboardController(model, view));
        view.setFocusable(true);
        view.requestFocus();
        frame.add(viewport);
        
        JToolBar toolbar = new JToolBar();
        toolbar.setFloatable(false);
        
        JButton smartBtn = new JButton("Smart");
        smartBtn.addActionListener(a -> {
        	model.setAction(null);
        	view.repaint();
        });
        toolbar.add(smartBtn);
        toolbar.addSeparator();
        
        for (GameAction action : possibleActions) {
            JButton moveBtn = new JButton(action.toString());
            moveBtn.addActionListener(a -> {
            	model.setAction(action);
            	view.repaint();
            });
            
            toolbar.add(moveBtn);
        }
        
        JButton turnDone = new JButton("done");
        turnDone.addActionListener(a -> {
        	model.done();
        	view.repaint();
        });
        toolbar.addSeparator();
        toolbar.add(turnDone);
        
        frame.add(toolbar, BorderLayout.SOUTH);
        
        frame.pack();
        frame.setVisible(true);

        UIController controller = new UIController(view, model);
        controller.addPossibleActions(possibleActions);
        
        return new UIController(view, model);
    }
}
